from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName


class AdView(BrowserView):
    """Shows a billboard ad."""

    template = ViewPageTemplateFile("ad.pt")

    def __call__(self):
        mtool = getToolByName(self.context, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        if not member.has_permission('Manage portal', self.context):
            self.request.set('disable_border', 1)
        return self.template()

    def can_edit(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Modify portal content', self.context)

    def get_elements(self):
        elements = []
        catalog = self.context.portal_catalog
        query = {
            'path': '/'.join(self.context.getPhysicalPath()),
            'portal_type': 'Image',
            }
        for brain in catalog(query):
            elements.append(brain.getURL())
        return elements

    def get_files(self):
        files = []
        catalog = self.context.portal_catalog
        query = {
            'path': '/'.join(self.context.getPhysicalPath()),
            'portal_type': 'File',
            }
        for brain in catalog(query):
            files.append(dict(
                    title=brain.Title,
                    url=brain.getURL()))
        return files



from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import setRoles, TEST_USER_ID, TEST_USER_NAME, login
from plone.testing import z2
from zope.configuration import xmlconfig


class FtwBillboardLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import ftw.billboard

        xmlconfig.file(
            'configure.zcml', ftw.billboard,
                context=configurationContext)

        # installProduct() is *only* necessary for packages outside
        # the Products.* namespace which are also declared as Zope 2 products,
        # using <five:registerPackage /> in ZCML.
        z2.installProduct(app, 'ftw.billboard')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.billboard:default')

        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)


FTW_BILLBOARD_FIXTURE = FtwBillboardLayer()
FTW_BILLBOARD_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_BILLBOARD_FIXTURE,), name="FtwBillboard:Integration")



from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.Archetypes import atapi
from Products.CMFCore.permissions import View
from Products.validation import V_REQUIRED
from Products.validation.config import validation
from Products.validation.validators.SupplValidators import MaxSizeValidator
from ftw.billboard import billboardMessageFactory as _
from ftw.billboard.config import PROJECTNAME
from ftw.billboard.interfaces import IBillboardCategory
from zope.interface import implements


validation.register(MaxSizeValidator('checkImageMaxSize',
                                     maxsize=zconf.ATImage.max_file_size))


BillboardCategorySchema = folder.ATFolderSchema.copy() + atapi.Schema((
        atapi.ImageField(
            name='image',
            required=False,
            primary=True,
            languageIndependent=True,
            storage=atapi.AnnotationStorage(),
            swallowResizeExceptions=zconf.swallowImageResizeExceptions.enable,
            pil_quality=zconf.pil_config.quality,
            pil_resize_algo=zconf.pil_config.resize_algo,
            max_size=zconf.ATImage.max_image_dimension,

            sizes={'large': (768, 768),
                   'preview': (400, 400),
                   'mini': (200, 200),
                   'thumb': (128, 128),
                   'tile': (64, 64),
                   'icon': (32, 32),
                   'listing': (16, 16),
                   },

            validators=(
                ('isNonEmptyFile', V_REQUIRED),
                ('checkImageMaxSize', V_REQUIRED)),
            widget=atapi.ImageWidget(
                label=_(u"label_image", default=u"Image"),
                show_content_type=False,
                )
            ),
        ))

# Set storage on fields copied from ATFolderSchema, making sure
# they work well with the python bridge properties.

schemata.finalizeATCTSchema(BillboardCategorySchema,
                            folderish=True,
                            moveDiscussion=False)


class BillboardCategory(folder.ATFolder):
    """A type for billboard categories."""
    implements(IBillboardCategory)
    schema = BillboardCategorySchema

    image = atapi.ATFieldProperty('image')

    security = ClassSecurityInfo()
    security.declareProtected(View, 'tag')

    def tag(self, **kwargs):
        """Generate image tag using the api of the ImageField
        """
        return self.getField('image').tag(self, **kwargs)

    def __bobo_traverse__(self, REQUEST, name):
        """Transparent access to image scales
        """
        if name.startswith('image'):
            field = self.getField('image')
            image = None
            if name == 'image':
                image = field.getScale(self)
            else:
                scalename = name[len('image_'):]
                if scalename in field.getAvailableSizes(self):
                    image = field.getScale(self, scale=scalename)
            if image is not None and not isinstance(image, basestring):
                # image might be None or '' for empty images
                return image

        return folder.ATFolder.__bobo_traverse__(self, REQUEST, name)

atapi.registerType(BillboardCategory, PROJECTNAME)



"""Common configuration constants
"""

PROJECTNAME = 'ftw.billboard'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'BillboardAd': 'ftw.billboard: Add BillboardAd',
    'BillboardCategory': 'ftw.billboard: Add BillboardCategory',
}

TINYMCE_ALLOWED_BUTTONS = (
    'bg-basicmarkup',
    'bold-button',
    'italic-button',
    'list-ol-addbutton',
    'list-ul-addbutton',
    'definitionlist',
    'linklibdrawer-button',
    'removelink-button',
    'tabledrawer-button'
)



from ftw.billboard import billboardMessageFactory as _
from Products.validation.interfaces.IValidator import IValidator
from Products.validation.i18n import recursiveTranslate
from Products.validation.i18n import safe_unicode
from zope.interface import implements
import re


class PhoneValidator:
    """Validates a phone number."""
    implements(IValidator)

    def __init__(self, name):
        self.name = name

    def __call__(self, value, *args, **kwargs):
        phone = re.compile('^(0|\+|00)[\d \'-/]{6,}$')
        if not phone.match(value):
            msg = _(
                "Validation failed($name: no valid phone number)",
                mapping={
                    'name': safe_unicode(self.name),
                    },
                )
            return msg
        return True


class MailValidator:
    """Validates an email address."""
    implements(IValidator)

    def __init__(self, name):
        self.name = name

    def __call__(self, value, *args, **kwargs):
        mail = re.compile("^(\w&.%#$&'\*+-/=?^_`{}|~]+!)*[\w&.%#$&'\*+-/=" +\
                              "?^_`{}|~]+@(([0-9a-z]([0-9a-z-]*[0-9a-z])?" +\
                              "\.)+[a-z]{2,6}|([0-9]{1,3}\.){3}[0-9]{1,3})$")
        if not mail.match(value):
            msg = _(
                "Validation failed($name: no valid email)",
                mapping={
                    'name': safe_unicode(self.name),
                    },
                )
            return recursiveTranslate(msg)
        return True



from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName


class CategoryView(BrowserView):
    """Shows a billboard category."""

    template = ViewPageTemplateFile("category.pt")

    def __call__(self):
        mtool = getToolByName(self.context, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        if not member.has_permission('Modify portal content', self.context):
            self.request.set('disable_border', 1)
        return self.template()

    def get_ads(self):
        """Returns all ads in this category."""
        context = aq_inner(self.context)
        ct = getToolByName(context, 'portal_catalog')

        # TODO: make it configurable if ads in all translated categories
        # should be shown.
        if not hasattr(context, 'getTranslations'):
            return ct(
                path='/'.join(self.context.getPhysicalPath()),
                portal_type='BillboardAd',
                sort_on='created',
                sort_order='descending',
                )
        else:
            translations = context.getTranslations().values()
            paths = ['/'.join(t[0].getPhysicalPath()) for t in translations]
            return ct(
                path=paths,
                portal_type='BillboardAd',
                Language='all',
                sort_on='created',
                sort_order='descending',
                )



from Acquisition import aq_inner
from ftw.billboard import billboardMessageFactory as _
from zope.interface import implements
from zope.component import getMultiAdapter
from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.portlets import base
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class IActionsPortlet(IPortletDataProvider):
    """Actions portlet interface.
    """


class Assignment(base.Assignment):
    implements(IActionsPortlet)

    @property
    def title(self):
        return _(
            u'label_billboardactions',
            default=u'Billboard Actions portlet')


class Renderer(base.Renderer):

    def __init__(self, *args, **kwargs):
        super(Renderer, self).__init__(*args, **kwargs)
        self.can_add = False
        self.can_edit = False
        self.can_del = False
        self.allowed_types = []

    def update(self):
        context = aq_inner(self.context)
        mtool = getMultiAdapter((context, self.request),
                         name=u'plone_tools').membership()

        self.can_add = mtool.checkPermission('ftw.billboard: Add BillboardAd',
            context) and context.portal_type == 'BillboardCategory'

        self.can_edit = mtool.checkPermission('Modify portal content',
            context) and context.portal_type == 'BillboardAd'

        self.can_del = mtool.checkPermission('Delete objects',
            context) and context.portal_type == 'BillboardAd'

        self.allowed_types = [term.id for term in
                             self.context.allowedContentTypes()]

    def available(self):
        if self.context.checkCreationFlag():
            return False
        return self.can_add or self.can_edit or self.can_del

    render = ViewPageTemplateFile('actions.pt')


class AddForm(base.NullAddForm):

    def create(self):
        return Assignment()



"""Definition of the BillboardAd content type
"""

from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.Archetypes import atapi
from Products.CMFCore.permissions import ManagePortal
from Products.validation.config import validation
from ftw.billboard import billboardMessageFactory as _
from ftw.billboard import validators
from ftw.billboard.config import PROJECTNAME
from ftw.billboard.config import TINYMCE_ALLOWED_BUTTONS
from ftw.billboard.interfaces import IBillboardAd
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.interface import implements


validation.register(validators.MailValidator('isEmail'))


BillboardAdSchema = folder.ATFolderSchema.copy() + atapi.Schema((
    atapi.TextField('text',
        searchable=True,
        required=False,
        allowable_content_types=('text/html',),
        default_content_type='text/html',
        validators=('isTidyHtmlWithCleanup',),
        default_output_type='text/x-html-safe',
        default_input_type='text/html',
        storage=atapi.AnnotationStorage(),
        widget=atapi.RichWidget(
            label=_(u"label_description", default=u"Description"),
            rows=15,
            allow_buttons=TINYMCE_ALLOWED_BUTTONS,
        ),
    ),

    atapi.FixedPointField(
        name='price',
        schemata='default',
        required=False,
        widget=atapi.DecimalWidget(
            label=_(u'label_price', default='price'),
            description=_(u'help_price', default='Leave blank '),
        ),
    ),

    atapi.StringField(
        name='contactMail',
        schemata='default',
        required=True,
        validators=('isEmail'),
        default_method='getDefaultContactMail',
        widget=atapi.StringWidget(
            label=_(u'label_contactmail', default='Contact E-Mail'),
            i18n_domain='ftw.billboard',
        ),
    ),

))

# Set storage on fields copied from ATFolderSchema, making sure
# they work well with the python bridge properties.

BillboardAdSchema['title'].searchable = True
BillboardAdSchema['title'].required = True
BillboardAdSchema['excludeFromNav'].default = True
BillboardAdSchema['expirationDate'].default_method = 'getDefaultExpirationDate'
BillboardAdSchema['effectiveDate'].default_method = 'getDefaultEffectiveDate'
BillboardAdSchema['description'].widget.visible = -1

# hide other schematas
BillboardAdSchema['allowDiscussion'].write_permission = ManagePortal
BillboardAdSchema['excludeFromNav'].write_permission = ManagePortal
BillboardAdSchema['nextPreviousEnabled'].write_permission = ManagePortal
BillboardAdSchema['subject'].write_permission = ManagePortal
BillboardAdSchema['relatedItems'].write_permission = ManagePortal
BillboardAdSchema['location'].write_permission = ManagePortal
BillboardAdSchema['language'].write_permission = ManagePortal
BillboardAdSchema['effectiveDate'].write_permission = ManagePortal
BillboardAdSchema['expirationDate'].write_permission = ManagePortal
BillboardAdSchema['creation_date'].write_permission = ManagePortal
BillboardAdSchema['modification_date'].write_permission = ManagePortal
BillboardAdSchema['creators'].write_permission = ManagePortal
BillboardAdSchema['contributors'].write_permission = ManagePortal
BillboardAdSchema['rights'].write_permission = ManagePortal


schemata.finalizeATCTSchema(BillboardAdSchema,
                            folderish=True,
                            moveDiscussion=False)


class BillboardAd(folder.ATFolder):
    """Billboard Ad"""
    security = ClassSecurityInfo()
    implements(IBillboardAd)

    portal_type = "BillboardAd"
    schema = BillboardAdSchema

    details = atapi.ATFieldProperty('details')

    def getDefaultExpirationDate(self):
        """Returns the default expiration date. The number of days the ad is
        public are defined in registry. If there is defined 0 days, there will
        be no default value."""
        registry = getUtility(IRegistry)
        days = registry.get('ftw.billboard.days_to_expire', 30)
        if days == 0:
            return None
        return DateTime() + days

    def getDefaultEffectiveDate(self):
        """Returns the default effective date (now) - appr. 5 min, because of
           limited accuracy of the DateTime widget
        """
        return DateTime() - 0.004

    def getDefaultContactMail(self):
        """Return the mail of the logged-in user"""
        member = self.portal_membership.getAuthenticatedMember()
        return member.getProperty('email', '')

    def getDefaultContactName(self):
        """Return the URL of the logged-in user"""
        member = self.portal_membership.getAuthenticatedMember()
        fullname = member.getProperty('fullname', '')
        firstname = member.getProperty('firstname', '')
        lastname = member.getProperty('lastname', '')
        if fullname:
            return fullname
        elif fullname or lastname:
            return ' '.join([firstname, lastname])
        return

atapi.registerType(BillboardAd, PROJECTNAME)



"""Main product initializer
"""

from Products.Archetypes import atapi
from Products.CMFCore import utils
from ftw.billboard import config
from zope.i18nmessageid import MessageFactory


billboardMessageFactory = MessageFactory('ftw.billboard')


def initialize(context):
    """Initializer called when used as a Zope 2 product.

    This is referenced from configure.zcml. Regstrations as a "Zope 2 product"
    is necessary for GenericSetup profiles to work, for example.

    Here, we call the Archetypes machinery to register our content types
    with Zope and the CMF.
    """

    content_types, constructors, _ftis = atapi.process_types(
        atapi.listTypes(config.PROJECTNAME),
        config.PROJECTNAME)

    for atype, constructor in zip(content_types, constructors):
        utils.ContentInit(
            '%s: %s' % (config.PROJECTNAME, atype.portal_type),
            content_types=(atype,),
            permission=config.ADD_PERMISSIONS[atype.portal_type],
            extra_constructors=(constructor,),
            ).initialize(context)



from ftw.billboard.portlets import actions
from ftw.billboard.testing import FTW_BILLBOARD_INTEGRATION_TESTING
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletRenderer
from zope.component import getUtility, getMultiAdapter
import unittest2 as unittest


class TestPortlet(unittest.TestCase):
    """ Basic tests for the recentlymodifiedportlet
    """

    layer = FTW_BILLBOARD_INTEGRATION_TESTING

    def testInterfaces(self):
        portlet = actions.Assignment()
        self.failUnless(
            actions.IActionsPortlet.providedBy(portlet))

    def testRenderer(self):
        context = self.layer['portal']
        request = self.layer['request']
        view = context.restrictedTraverse('@@plone')
        manager = getUtility(
            IPortletManager, name='plone.rightcolumn', context=context)
        assignment = actions.Assignment()

        renderer = getMultiAdapter(
            (context, request, view, manager, assignment), IPortletRenderer)

        self.failUnless(isinstance(renderer, actions.Renderer))



from setuptools import setup, find_packages
import os


version = '1.1.8'
maintainer = 'Julian Infanger'


tests_require = [
    'zope.testing',
    'plone.app.testing',
    'plone.mocktestcase',
    ]


setup(name='ftw.billboard',
      version=version,
      description='Simple market place addon for plone.',
      long_description=open('README.rst').read() + '\n' + \
          open(os.path.join('docs', 'HISTORY.txt')).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.1',
        'Framework :: Zope2',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='plone billboard ads advertisement ftw',
      author='4teamwork GmbH',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/ftw.billboard',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'plone.testing',
        'setuptools',
        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),

      entry_points='''
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      ''',
      )



import os
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from ftw.billboard import billboardMessageFactory as _
from zope.app.container.interfaces import INameChooser


class AddFile(BrowserView):
    """Form to upload a file"""

    template = ViewPageTemplateFile("simple_upload.pt")
    allowed_extensions = ['.pdf', '.doc', '.docx', '.xls', '.xlsx']
    create_type = 'File'
    field_name = 'file'
    view_name = 'add_file'

    def __call__(self):
        self.request.set('disable_border', 1)
        if self.request.form.get('form.submitted', None):
            status = IStatusMessage(self.context.REQUEST)
            upload_file = self.request.form.get('upload_file', None)
            error = self.is_wrong_type(upload_file)
            if error:
                status.addStatusMessage(error, type='error')
                return self.template()
            chooser = INameChooser(self.context)
            file_id = chooser.chooseName(upload_file.filename, self)
            self.context.invokeFactory(self.create_type,
                                       file_id,
                                       title=upload_file.filename)
            new_file = self.context.get(file_id)
            new_file.getField(self.field_name).set(new_file, upload_file)
            return self.request.response.redirect('.')
        return self.template()

    def is_wrong_type(self, upload):
        if not upload:
            return _(u'label_required', default=u'Required field')
        else:
            _root, extension = os.path.splitext(upload.filename)
            if extension not in self.allowed_extensions:
                return _(
                    u'label_notallowedtype',
                    default=u'Not allowed type (${types})',
                    mapping={u'types': ', '.join(self.allowed_extensions)})
        return False


class AddImage(AddFile):
    """Form to upload an image"""

    allowed_extensions = ['.jpg', '.jpeg', '.png', '.gif']
    create_type = 'Image'
    field_name = 'image'
    view_name = 'add_image'
